/*
Copyright ＿ 2005, Apple Computer, Inc.  All rights reserved.
NOTE:  Use of this source code is subject to the terms of the Software
License Agreement for Mac OS X, which accompanies the code.  Your use
of this source code signifies your agreement to such license terms and
conditions.  Except as expressly granted in the Software License Agreement
for Mac OS X, no other copyright, patent, or other intellectual property
license or right is granted, either expressly or by implication, by Apple.
*/

var allTheData = [{symbol:'^DJI', exchange:"", name:"DOW JONES INDUSTRIAL AVERAGE", lasttrade:0, change:0, pctchange:0},
				 {symbol:'^IXIC', exchange:"", name:"NASDAQ COMPOSITE", lasttrade:0, change:0, pctchange:0},
				 {symbol:'AAPL', exchange:"", name:"Apple Computer", lasttrade:0, change:0, pctchange:0},
				 {symbol:'EBAY', exchange:"", name:"eBay", lasttrade:0, change:0, pctchange:0},
				 {symbol:'GOOG', exchange:"", name:"Google Inc.", lasttrade:0, change:0, pctchange:0},
				 {symbol:'AMZN', exchange:"", name:"Amazon.com, Inc.", lasttrade:0, change:0, pctchange:0}
				];
var symbolData = new Array();
var editRolloverImg = null;
var showingCharts = true;
				  				  
var currentSelection = null;
var anythingChangedOnBackside = false;
var currentChartMode = '6m';
var isShowingFront = true;
var allMarketsOpen = true;
var timer = null;
var removeButton = null;
var lastDownloadDate = null;

var kChartCanvasWidth = 144;
var kChartCanvasHeight = 48;

function getLocalizedString (key)
{
	try {
		var ret = localizedStrings[key];
		if (ret === undefined)
			ret = key;
		return ret;
	} catch (ex) {}

	return key;
}

function getPrettySymbol (symbol)
{
	return symbol;
}

function updateStatusText()
{
	if (lastDownloadDate != null)
	{
		// Update our "last updated" timer
		var curDate = new Date();
		var dateDiff = curDate.getTime() - lastDownloadDate.getTime();
		dateDiff = Math.round(dateDiff / 1000); // to seconds
				
		if (allMarketsOpen)
		{
			if (dateDiff < 60)
			{
				document.getElementById('delayed').innerText = getLocalizedString("Quotes just updated");
			}
			else if (dateDiff < 120)
			{
				document.getElementById('delayed').innerText = getLocalizedString("Quotes last updated one minute ago");
			}
			else if (dateDiff < 3600)
			{
				document.getElementById('delayed').innerText = getLocalizedString("Quotes last updated %@ minutes ago").replace("%@", Math.round(dateDiff / 60));
			}
			else if (curDate.getDate() == lastDownloadDate.getDate() &&
					 curDate.getMonth() == lastDownloadDate.getMonth() &&
					 curDate.getFullYear() == lastDownloadDate.getFullYear()) // same day
			{
				document.getElementById('delayed').innerText = getLocalizedString("Quotes last updated at %@").replace("%@", lastDownloadDate.toLocaleTimeString("short"));
			}
			else // different day
			{
				document.getElementById('delayed').innerText = getLocalizedString("Quotes last updated on %@").replace("%@", lastDownloadDate.toLocaleDateString("short"));
			}
		}
		else // markets closed
		{
			if (dateDiff < 3600) // simplify more common case, also better close to day changeover
			{
				document.getElementById('delayed').innerText = getLocalizedString("Markets closed (%@)").replace("%@", lastDownloadDate.toLocaleTimeString("short"));
			}
			else if (curDate.getDate() == lastDownloadDate.getDate() &&
					 curDate.getMonth() == lastDownloadDate.getMonth() &&
					 curDate.getFullYear() == lastDownloadDate.getFullYear()) // same day
			{
				document.getElementById('delayed').innerText = getLocalizedString("Markets closed (%@)").replace("%@", lastDownloadDate.toLocaleTimeString("short"));
			}
			else // different day
			{
				document.getElementById('delayed').innerText = getLocalizedString("Markets closed (%@)").replace("%@", lastDownloadDate.toLocaleDateString("short"));
			}
		}
	}
	else
	{
		document.getElementById('delayed').innerText = getLocalizedString("Retrieving Data…"); // we just started with no data
	}
}

function xml_callback(object)
{
	if (!object.error)
	{
		var c = object.quotes.length;
  		for (var i = 0; i < c; ++i)
  		{
  			try 
  			{
				var quote = object.quotes[i];
				
				// Ensure we attempt to convert to numbers
				var change = new Number(quote.change);
				var pctchange = new Number(quote.pctchange);
				var lasttrade = new Number(quote.lasttrade);
				
				var showpercent = getShowPercent();
				symbolData[i] = quote;
				
				// round to 2 decimal places. These can be returned as "N/A", so grab the string if that happens
				change = isFinite(change) ? change.toFixed(2) : new String(quote.change);
				pctchange = isFinite(pctchange) ? pctchange.toFixed(2) : new String(quote.pctchange);
				lasttrade = isFinite(lasttrade) ? lasttrade.toFixed(2) : new String(quote.lasttrade);

				var change_div = document.getElementById(quote.symbol+"_change");

				if(!change_div) //if that didnt work, it might be an index's alternate symbol
				{
					quote.symbol = ("^IXIC" == quote.symbol) ? "COMPX" : quote.symbol;
					quote.symbol = ("^DJI" == quote.symbol) ? "INDU" : quote.symbol;					
					change_div = document.getElementById(quote.symbol+"_change");	
				}
				
				var last_trade_span = document.getElementById(quote.symbol+"_lastTrade");
				var percent_div = document.getElementById(quote.symbol+"_percent");
				var background_icon = document.getElementById (quote.symbol+"_icon");
				
				if (change < 0 || pctchange < 0)
				{
					background_icon.style.backgroundImage = "url(Images/down.png)";
					change = change.substr(1);
					pctchange = pctchange.substr(1);
				}
				else
				{
					// percentages can comeback -0.00.  strip the negative in this case.
					if (pctchange[0] == "-")
						pctchange = pctchange.substr(1);
					background_icon.style.backgroundImage = "url(Images/up.png)";
				}
				
				//
				// [ouch] position dependent code (firstChild).  Not the best.
				//
				last_trade_span.innerText = lasttrade;
				change_div.innerText = change;
				percent_div.innerText = pctchange+"%";
				
				if (!showpercent)
				{
					change_div.style.fontSize = "13px";
					changeFontOfElement(change_div, 41);
				}
				else
				{
					percent_div.style.fontSize = "12px";
					changeFontOfElement(percent_div, 41);
				}
				
				// Store last downloaded time
				lastDownloadDate = new Date();
			
			} catch (ex)
			{
				// dont let one exception lose all the data
				alert(ex.name + ". " + ex.message);
			}
		}
		
		if (object.open != allMarketsOpen)
		{
			allMarketsOpen = object.open;
		}
	}
	else
		alert ("Stocks Widget: XMLHttpRequest Failed (" + object.errorString + ")");
	
	// Regardless of what happened above
	updateStatusText(); // "Quotes just updated" or "Last updated xx minutes ago"
}


function createRow (object)
{
	var theSymbol = object.symbol;
	var showPercent = getShowPercent();
	var prettySymbol = getPrettySymbol(theSymbol);
				
	var row = document.createElement('tr');
	
	row.setAttribute ("class", "stockRow");
	row.setAttribute ("onclick", "clickonrow(event, this);");
	row.setAttribute ("ondblclick", "clickedOnSymbol(this.getAttribute('tag'), this.getAttribute('stock_exchange'));");
	row.setAttribute ("tag", theSymbol);
	row.setAttribute ("stock_exchange", object.exchange);
	
	var td = document.createElement ('td');
	td.setAttribute ("class", "symbol");
	td.appendChild (document.createTextNode(prettySymbol));
	row.appendChild (td);
	
	td = document.createElement('td');
	td.setAttribute ("class", "value");
	td.setAttribute ("id", theSymbol + "_lastTrade");
	row.appendChild (td);
	
	td = document.createElement('td');
	var div = document.createElement('div');
	div.setAttribute ("class", "updown");
	div.setAttribute ("id", theSymbol + "_icon");
	div.setAttribute ("onclick", "showAsPercentChanged(event);");
	
	var table = document.createElement('table');
	table.setAttribute ('class', 'inner-table');
	table.setAttribute ('cellspacing', '0');
	table.setAttribute ('cellpadding', '0');
	
	var tbody = document.createElement ('tbody');
	var tr = document.createElement ('tr');
	var innertd = document.createElement ('td');
	var innerdiv = document.createElement ('div');
	
	innerdiv.setAttribute ('class', showPercent ? 'change hidden' : 'change');
	innerdiv.setAttribute ('id', theSymbol + "_change");
	innertd.appendChild(innerdiv);
	
	innerdiv = document.createElement ('div');
	innerdiv.setAttribute ('class', showPercent ? 'change' : 'change hidden');
	innerdiv.setAttribute ('id', theSymbol + "_percent");
	innertd.appendChild(innerdiv);

	tr.appendChild (innertd);
	tbody.appendChild(tr);
	table.appendChild(tbody);
	div.appendChild (table);
	td.appendChild (div);
	row.appendChild (td);
		
	return row;
}

var showPercent = true;
var showPercentFetched = false;
function getShowPercent(forceFetch)
{
	if (forceFetch || !showPercentFetched)
	{
		if (window.widget)
		{
			var pref = null;
			// this is crashing TOT WebKit.  It should not but it is. turning off for now.
			var pref = widget.preferenceForKey("show-percent");
			
			if (pref != null && pref.length > 0)
				showPercent = true;
			else
				showPercent = false;
		}
		showPercentFetched = true;	
	}
	
	return showPercent;
}

function setShowPercent(show_percent)
{
	showPercent = show_percent;
	if (window.widget)
	{
		var pref = show_percent ? "true" : "";
		widget.setPreferenceForKey(pref, "show-percent");
	}
}

function showbackside(event) {
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	resizedWhenFlippedOver = false;
	if (window.widget)
	{
		if (calculateWidgetHeight() < 277)
		{
			window.resizeTo (213, 277);
			resizedWhenFlippedOver = true;
		}
		widget.prepareForTransition("ToBack");
	}
	
	front.style.display="none";
	back.style.display="block";
	isShowingFront = false;
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);
	
	anythingChangedOnBackside = false;
	
	if (timer != null)
	{
		clearInterval (timer);
		timer = null;
	}

	document.removeEventListener("keypress", keyPressed, true);
	document.addEventListener("keypress", backsideKeyPressed, true);
	
	selectAllInSymbolTextField();

	event.stopPropagation();
	event.preventDefault();
}


var resizedWhenFlippedOver = false;
function selectDone ()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget)
		widget.prepareForTransition("ToFront");
	
	front.style.display="block";
	back.style.display="none";
	
	isShowingFront = true;
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);
		
	if (anythingChangedOnBackside)
	{
		// fix up widget
		var lastSelected = null;
		
		if (currentSelection != null)
			lastSelected = currentSelection.getAttribute("tag");
			
		setupMainTable (false);
		
		if (lastSelected != null)
		{
			var c = allTheData.length;
			var found = false;
			for (var i = 0; i < c; ++i)
			{
				if (allTheData[i].symbol == lastSelected)
				{
					found = true;
					break;
				}
			}
			
			if (!found)
			{
				// the item we have selected is gone. select the first one
				clickonrowimp (document.getElementById('stocksHeader').firstChild.firstChild);
			}
			else
			{
				moveSelectionToTag (lastSelected);
			}
		}
		
		if (window.widget)
		{
			resizedWhenFlippedOver = true; // good chance it will need to be resized
			
			// save of the new prefs
			
			var pref = createAllDataJSString();
			setPreferenceForKey (pref, "symbols");
		}
	}
	
	onshow();

	if (resizedWhenFlippedOver)
	{
		window.resizeTo(213, calculateWidgetHeight());
		resizedWhenFlippedOver = false;
	}

	document.removeEventListener("keypress", backsideKeyPressed, true);
	document.addEventListener("keypress", keyPressed, true);
}

function generateSymbolString (object)
{
		return object.symbol;
}

function generateSymbolStringFromStrings (symbol, exchange)
{
		return symbol;
}

function getStockData()
{
	var c = allTheData.length;

	if (c > 0)
	{
		var symbols = [];

		for (var i = 0; i < c; ++i)
		{
			symbols.push(generateSymbolString(allTheData[i]));
		}
	
		fetchStockData (xml_callback, symbols);
		
		//
		// ouch should only do this if the day is not close, otherwise
		// there is no need to update.  get this info in the feed somewhow
		//
		if (currentSelection != null && currentChartMode == '1d')
		{
			if (lastChartRequest != null)
				lastChartRequest.abort();
			var tag = currentSelection.getAttribute("tag");
			var exchange = currentSelection.getAttribute("stock_exchange");
			lastChartRequest = fetchChartData (generateSymbolStringFromStrings(tag, exchange), currentChartMode, chart_callback);
		}
	}
}

function changeFontOfElement(element, width)
{
    var elementWidth = 99999;

    while (elementWidth > width) {

        var style = document.defaultView.getComputedStyle(element,'');
        		
		if (!style)
		{
			break;
		}
		
        var fontSize = parseInt(style.getPropertyValue("font-size"));

        elementWidth = parseInt(style.getPropertyValue("width"));
        if (elementWidth > width)
        {
    
			element.style.fontSize = (fontSize - 1) + "px";
		
			style = document.defaultView.getComputedStyle(element,'');
			elementWidth = parseInt(style.getPropertyValue("width"));
		}
		else
		{
			break;
		}
    }
}

var img = new Image();
img.src = "Images/selection-middle.png";
img.src = "Images/yahoo_hover.png";
img.src = "Images/yahoo_down.png";

function moveSelectionToTag (tag)
{
	var index;
	var offset=1;
	var c = allTheData.length;
	for (index=0; index < c; ++index)
	{
		if (allTheData[index].symbol == tag)
			break;
	}
	if (index == c) index = 0;
	
	switch (index)
	{
		case 0:
			selection.src = "Images/selection-first.png";
			offset = 0;
			break;
		case c - 1:
			selection.src = "Images/selection-last.png";
			break;
			
		default:
			selection.src = "Images/selection-middle.png";
			break;
	}
	
	// top is 7px;
	// had to add an offset. The top is at 7px, but the others needed
	// to go down by one pixel
	selection.style.top = (25*index+7+offset).toString() + "px";
	selection.style.display = "block";
}

function clickonrowimp (row)
{
	var selection = document.getElementById('selection');
			
	currentSelection = row;
	
	moveSelectionToTag (row.getAttribute("tag"));
		
	getChartForCurrentMode ();
}

function setupChartAnimation (hiding)
{
	var chartDiv = document.getElementById('chart');
	var timeNow = (new Date).getTime();
	var multiplier = (event.shiftKey ? 10 : 1); // enable slo-mo
	var startingSize = parseInt(chartDiv.clientHeight,10);

	resizeAnimation.element = chartDiv;
	if (resizeAnimation.timer != null) // it is moving... change to new size
	{
		clearInterval(resizeAnimation.timer);
		resizeAnimation.timer = null;
		resizeAnimation.duration -= (timeNow - resizeAnimation.startTime);
		resizeAnimation.positionFrom = resizeAnimation.positionNow;
	}
	else
	{
		resizeAnimation.duration = 250 * multiplier;
		resizeAnimation.positionFrom = startingSize;
	}
	
	if (window.widget && !hiding)
	{
		window.resizeTo (213, calculateWidgetHeight());
	}
	resizeAnimation.positionTo = hiding ? 0 : 80;
	resizeAnimation.startTime = timeNow - 13; // set it back one frame.
	resizeAnimation.onfinished = hiding ? animFinished : null;
	
	resizeAnimation.element.style.height = startingSize;
	resizeAnimation.timer = setInterval ("animate();", 13);
	animate();
}

function clickonrow (event, row)
{
	var oldSelection = currentSelection;
	
	if (currentSelection != null && row == currentSelection)
	{
		document.getElementById('selection').style.display = "none";
		currentSelection = null;
		if (window.widget)
			widget.setPreferenceForKey ("!!NONE!!", createKey("selection"));
	
		setupChartAnimation (true);
		showingCharts = false;
		if (lastChartRequest != null)
		{
			lastChartRequest.abort();
			lastChartRequest = null;
		}
	}
	else
	{
		clickonrowimp(row);
		
		if (window.widget && oldSelection != currentSelection)
		{
			//write out the selection
			widget.setPreferenceForKey (row.getAttribute("tag"), createKey("selection"));
		}
		
		if (oldSelection == null)
		{
			showingCharts = true;
			setupChartAnimation (false);
		}
	}
}

function updateChangeSpans ()
{
	var c = symbolData.length;
	var change_span;
	var size;
	var type;
	
	if (getShowPercent())
	{
		size = "12px";
		type = "_percent";
	}
	else
	{
		size = "13px";
		type = "_change";
	}
	
	for (var i = 0; i < c; ++i)
	{
		change_span = document.getElementById(symbolData[i].symbol+type);
		change_span.style.fontSize = size;
		changeFontOfElement(change_span, 41);
	}
}

function showAsPercentChanged (event)
{
	clickonchange(event);
	event.stopPropagation();
	event.preventDefault();
}

function updatePercentCheckbox (showPercent)
{
	var checkbox = document.getElementById('percentcheck');	
	checkbox.checked = showPercent;
}

function clickonchange (event)
{
	var showPercent = getShowPercent();

	var newVal = !showPercent;
	// set the preference
	setShowPercent (newVal);
	updateShowPercentUI(newVal);

	// stop proagation otherwise the clickonrow handler would be called
	event.stopPropagation();
}

function updateShowPercentUI (showPercent) {
	var c = symbolData.length;
	var i;
	var change;
	var percent;
	
	if (!showPercent) // show change
	{
		for (i = 0; i < c; ++i)
		{
			change = document.getElementById(symbolData[i].symbol+"_change");
			percent = document.getElementById(symbolData[i].symbol+"_percent");
			
			percent.setAttribute ('class', 'change hidden');
			change.setAttribute ('class', 'change');
			
		}
	}
	else // show percent
	{
		for (i = 0; i < c; ++i)
		{

			change = document.getElementById(symbolData[i].symbol+"_change");
			percent = document.getElementById(symbolData[i].symbol+"_percent");

			percent.setAttribute ('class', 'change');
			change.setAttribute ('class', 'change hidden');
		}
	}
	
	updateChangeSpans();
	updatePercentCheckbox(showPercent);
}

function clickedOnSymbol (symbol, exchange)
{
	if (window.widget)
	{
		symbol = ("COMPX" == symbol) ? "^IXIC" : symbol;
		var url = 'http://api.apple.go.yahoo.com/appledwf/q?s=' + symbol;
		widget.openURL(url);
	}
}

function setupFetchTimer ()
{
	timer = setInterval('getStockData();', 120000);

}

function onshow ()
{
	if (timer == null)
	{
		getStockData(); // reload we were just shown
		timer = setInterval('getStockData();', 120000);
	}
	else
	{
		updateStatusText(); // at least update them on our status
	}
}

function onhide ()
{
	if (timer != null)
	{
		// we were hidden clear the timer.
		clearInterval(timer);
		timer = null;
	}
}

document.addEventListener("keypress", keyPressed, true);


function keyPressed(e) {
	
	var handled = true;
	var selection = currentSelection;
	
	if (selection != null)
	{
		switch (e.charCode)
		{
			case 63232: // up
				selection = selection.previousSibling;
				if (selection == null)
				{
					selection = document.getElementById('stocksHeader').firstChild.lastChild;
				}
				break;
				
			case 63233: // down
				selection = selection.nextSibling;
				if (selection == null)
				{
					selection = document.getElementById('stocksHeader').firstChild.firstChild;
				}
				break;
				
			default:
				handled = false;
				break;
		}
		
		if (handled)
		{
			clickonrow (e, selection);
			
			e.stopPropagation();
			e.preventDefault();
		}
	}
}

function setupMainTable(addSymbolsToBackside)
{
	var c = allTheData.length;
	var container = document.getElementById('tbody');
	removeAllChildren(container);		

	if (addSymbolsToBackside) {
		var innerList = document.getElementById('inner-list');
		innerList.innerHTML = "";
	}

	for (var i=0; i<c; ++i)
	{
		var data = allTheData[i];
		var row = createRow (data);
		container.appendChild (row);
		if (addSymbolsToBackside)
			addsymbol (getPrettySymbol(data.symbol), data.name);
	}
	
	//set the bottom of table to correct class
	if (c < 2) c = 2;
	document.getElementById('tableBottom').setAttribute ("class", (c%2) != 0 ? "tableBottomDark" : "tableBottomLight");
	
	// fill in the backgorund
	container = document.getElementById('tableContainer');
	removeAllChildren(container);		

	c = c - 2;
	var light = true;
	for (var i = 0; i < c; ++i)
	{
		var div = document.createElement ('div');
		div.setAttribute  ("class", light ? "tableRowLight" : "tableRowDark");
		container.appendChild (div);
		light = !light;
	}
}

function loadPreferences() {
	if (window.widget)
	{
		try {
			selectedSymbol = getPreferenceForKey("selection", false);
			var symbols = getPreferenceForKey ("symbols", true);
			var newAllData = new Array;
			
			symbols = eval(symbols);
			if (!symbols || !symbols.length)
				return;

			var c = symbols.length;
			
			if (c == 0)
			{
				allTheData = new Array; // nothing to see move along
			}
			else
			{
				for (var i = 0; i < c; ++i)
				{
					var data = symbols[i];
					
					var exchange = data.exchange === undefined ? '' : data.exchange;
					newAllData[newAllData.length] = {symbol:data.symbol, exchange:exchange, name:data.name, lasttrade:0, change:0, pctchange:0};
				}
				
				allTheData = newAllData;
			}
			
			currentChartMode = getPreferenceForKey("chartmode", false);
			if (currentChartMode == null) currentChartMode='6m';
		
		} catch(ex)
		{
			alert(ex.name + ". " + ex.message);
		}
	}
}

function setupUI() {
	// add all of our stocks to the widget
	setupMainTable (true);
	setupChartController (currentChartMode);

	var header = document.getElementById('tbody');
	currentSelection = header.firstChild;
	if (selectedSymbol != null)
	{
		if (selectedSymbol == '!!NONE!!')
		{
			document.getElementById('selection').style.display = "none";
			currentSelection = null;
			showingCharts = false;
			document.getElementById('chart').style.height = "0";
		}
		else
		{
			for (var child = header.firstChild; child != null; child = child.nextSibling)
			{
				if (child.getAttribute("tag") == selectedSymbol)
				{
					// no need to do anything if the first item is the one selected.
					if (currentSelection != child)
					{
						clickonrowimp (child);
					}
					break;
				}
			}
		}
	}
	else
	{
		// try to select apple.
		for (var child = header.firstChild; child != null; child = child.nextSibling)
		{
			if (child.getAttribute("tag") == 'AAPL')
			{
				// no need to do anything if the first item is the one selected.
				if (currentSelection != child)
				{
					clickonrowimp (child);
				}
				break;
			}
		}
	}

	if (showingCharts)
	{
	 	getChartForCurrentMode();
	}
	
	if (getShowPercent())
	{
		var element = document.getElementById('percentcheck');	
		element.checked = true;
	}
	
	if (window.widget && (allTheData.length != 6 || !showingCharts))
	{
		window.resizeTo (213, calculateWidgetHeight());
	}
}

function setupWidget()
{
	var selectedSymbol = null;

	loadPreferences();
	setupUI();
	
	// front side strings
	// this will happen every time it is shown.
	//document.getElementById('retrieving').innerText = getLocalizedString('Retrieving Data…');
	
	// backside strings
	document.getElementById('quoteslabel').innerText = getLocalizedString('Quotes:');
	document.getElementById('percenttext').innerHTML = getLocalizedString('Show change as a percentage');
	document.getElementById('tosLink').innerText = getLocalizedString('Terms of Service');
	
	// Info button
	new AppleInfoButton(document.getElementById('flip'), document.getElementById('front'),
						"white", "white", showbackside);
	
	// Remove button
	removeButton = new AppleGlassButton(document.getElementById('remove'), 
										getLocalizedString('Remove'), removeButtonClicked);
	removeButton.setEnabled(false);
	
	// Done button
	var doneButton = new AppleGlassButton(document.getElementById('done'),
										  getLocalizedString('Done'), selectDone);
	doneButton.textElement.style.textAlign = "center";
	doneButton.textElement.style.width = "47px";
	
	onshow();
}

function onsync() {
	loadPreferences();
	setupUI();
	updateShowPercentUI(getShowPercent(true));
	onshow();
}

// add the onhide and onshow callbacks
if (window.widget)
{
	widget.onhide = onhide;
	widget.onshow = onshow;
	widget.onremove = onremove;
	widget.onsync = onsync;
}

var selectedRow=null;
function clickonbackrow(event, row)
{
	if (allTheData.length > 1)
		removeButton.setEnabled(true);

	if (selectedRow != row)
	{
		if (selectedRow != null)
			selectedRow.setAttribute("class", "row");
		selectedRow = row;
		
		row.setAttribute ("class", "row select");
	}	
	
	if (event)
	{
		event.stopPropagation();
		event.preventDefault();
	}
}

function backsideKeyPressed(event)
{
	var newSelection = null;
	switch (event.keyCode)
	{
		case 63232: // up
			if (selectedRow != null)
				newSelection = selectedRow.previousSibling;
			if (newSelection == null)
				newSelection = document.getElementById('inner-list').lastChild;
			clickonbackrow(null, newSelection);
			break;
			
		case 63233: // down
			if (selectedRow != null)
				newSelection = selectedRow.nextSibling;
			if (newSelection == null)
				newSelection = document.getElementById('inner-list').firstChild;
			clickonbackrow(null, newSelection);
			break;
	}
	
	if (event)
	{
		event.stopPropagation();
		event.preventDefault();
	}
}

function symbolFocus(event)
{
	document.removeEventListener("keypress", backsideKeyPressed, true);
}

function symbolBlur(event)
{
	document.addEventListener("keypress", backsideKeyPressed, true);
}

var validateTimerData = null;
var lastValidEntry = null;
var addButtonClicked = false;

function symbolChanged (event)
{	
	var symbol = document.getElementById('symbolinput').value;
		
	if (symbol != null && symbol.length > 0 && addButtonClicked)
	{
		var request = validateSymbol (validateSymbolCallback, symbol);
		
		cancelValidateTimer();
		lastValidEntry = null;

		var validate = document.getElementById('validate');
		validate.innerText = getLocalizedString('Validating');
		validate.style.display = "block";
		
		validateTimerData = {timer:setInterval('validateTimer();', 500), pos:0, request:request};
	}
}

function symbolTyping (event)
{
	// if we are typing in the input then we cannnot have just clicked the add button
	addButtonClicked = false;
	lastValidEntry = null;
}

function symbolKeyPress (event)
{
	switch (event.keyCode)
	{
		case 13: // return
		case 3:  // enter
			addButtonClicked = true;
			break;
	}
	
	if (addButtonClicked)
	{
		if (lastValidEntry != null)
			addOrSelectItem ();
		else
			symbolChanged();
	}
}

function cancelValidateTimer ()
{
	if (validateTimerData != null)
	{
		validateTimerData.request.abort();
		clearInterval(validateTimerData.timer);
		validateTimerData = null;
		document.getElementById('validate').style.display="none";
	}
}

function validateTimer ()
{
	var validate = document.getElementById('validate');
	
	// position goes 0 -> 3 and then resets
	validateTimerData.pos++;
	
	if (validateTimerData.pos > 3)
	{
		validateTimerData.pos = 0;
	}
	
	var text = getLocalizedString('Validating');
	for (var i = 0; i < validateTimerData.pos; ++i)
		text+= '.';
	
	validate.innerText = text;
}

function validateSymbolCallback (object)
{
	var menu = null;
	var inputChanged = false;
	
	cancelValidateTimer();
	
	if (isShowingFront) return;
	
	if (window.widget)
		menu = widget.createMenu();

	if (object.error)
	{
		if (menu != null)
		{
			menu.addMenuItem (getLocalizedString("Failed to validate symbol"));
			menu.setMenuItemEnabledAtIndex (0, false);	
		}
		alert ('Validating symbol failed, error=' + object.errorString);
	}
	else if (object.symbols.length < 1)
	{
		if (menu != null)
		{
			menu.addMenuItem (getLocalizedString("No matching symbols"));
			menu.setMenuItemEnabledAtIndex (0, false);	
		}
	}
	else
	{
		if (menu == null || object.symbols.length == 1) // there is only one, just put it in
		{
			menu = null;

			lastValidEntry = object.symbols[0];
			document.getElementById('symbolinput').value = lastValidEntry.symbol;
			inputChanged = true;
		}
		else
		{
			if (menu != null)
			{
				var c = object.symbols.length;
				var a = object.symbols;
				var maxLength = 0;
							
				var lastExchange = a[0].exchange;
				var lastIndex = 0;
				for (var i = 0; i < c; ++i)
				{
					var str = a[i].symbol;
					var name = a[i].name;
					var exchange = a[i].exchange;
					
					if (name != null || exchange != null)
					{
						str += " - "	;			
					
						if (name != null)
							str += name;
						
						if (exchange != null)
						{
							if (name != null)
								str += " (";
							str += exchange;
							if (name != null)
								str += ")";
						}
					}
					
					if (exchange != null && exchange != lastExchange)
					{
						menu.addSeparatorMenuItem();
						lastIndex++;
						lastExchange = exchange;
					}
					menu.addMenuItem (str);
					menu.setMenuItemTagAtIndex (lastIndex, i+1); // switch to one based
					lastIndex++;
				}
			}
		}
	}
	
	if (menu != null)
	{
		var selectedItem = menu.popup (15, 43);
		
		if (selectedItem >= 0)
		{
			var i = menu.getMenuItemTagAtIndex(selectedItem) - 1;
			lastValidEntry = object.symbols[i];
			document.getElementById('symbolinput').value = lastValidEntry.symbol;
			inputChanged = true;
		}	
	}
	
	if (addButtonClicked && inputChanged)
	{
		addOrSelectItem ();	
	}
	
	addButtonClicked = false;
}

function symbolAlreadyInSet(symbol)
{
	var c = allTheData.length;
	
	for (var i = 0; i < c; ++i)
	{
		//check for alternate symbol (COMPX/INDU only seen as defaults, can't be added)
		if(symbol == "^IXIC" && allTheData[i].symbol == "COMPX" ||
			symbol == "^DJI" && allTheData[i].symbol == "INDU" ||
			allTheData[i].symbol == symbol)
			return i;
	}

	return -1;
}

function addOrSelectItem ()
{
	var index = symbolAlreadyInSet(lastValidEntry.symbol)
	if (index < 0)
	{
		// make sure we have the correct number of items
		if (allTheData.length < 20)
		{
			var row = addsymbol (lastValidEntry.symbol, lastValidEntry.name);
		
			allTheData[allTheData.length] = {symbol: lastValidEntry.symbol, exchange:lastValidEntry.exchange, name:lastValidEntry.name, lasttrade:0, change:0, pctchange:0};
			anythingChangedOnBackside = true;
			
			clickonbackrow (null, row);
		}
	}
	else
	{
		// select the item that is already there.
		var row = document.getElementById('inner-list').firstChild;
		for (var i = 0; i < index; ++i)
			row = row.nextSibling;
			
		if (row != null) // just to be safe, should never happen
		{
			clickonbackrow (null, row);
		}
	}

	addButtonClicked = false;
}

function addbuttonclicked (event)
{
	addButtonClicked = true;
	if (lastValidEntry == null)
		symbolChanged();
	else
		addOrSelectItem();
}
var mouseDownInButton = false;
function addButtonMouseDown (event)
{
	var button = document.getElementById('addbutton');
	
	button.src = 'Images/addbuttondown.png';

	document.addEventListener("mousemove", addButtonMouseMove, true);
	document.addEventListener("mouseup", addButtonMouseUp, true);
	button.addEventListener("mouseover", addButtonMouseOver, true);
	button.addEventListener("mouseout", addButtonMouseOut, true);	
	
	mouseDownInButton = true;
	
	event.stopPropagation();
	event.preventDefault();
}

function addButtonMouseMove (event)
{
	event.stopPropagation();
	event.preventDefault();
}

function addButtonMouseOver (event)
{
	mouseDownInButton = true;
	document.getElementById('addbutton').src = 'Images/addbuttondown.png';
	event.stopPropagation();
	event.preventDefault();
}

function addButtonMouseOut (event)
{
	document.getElementById('addbutton').src = 'Images/addbutton.png';

	mouseDownInButton = false;

	event.stopPropagation();
	event.preventDefault();

}

function addButtonMouseUp (event)
{
	var button = document.getElementById('addbutton');

	document.removeEventListener("mousemove", addButtonMouseMove, true);
	document.removeEventListener("mouseup", addButtonMouseUp, true);	
	button.removeEventListener("mouseover", addButtonMouseOver, true);
	button.removeEventListener("mouseout", addButtonMouseOut, true);

	button.src = 'Images/addbutton.png';
	if (mouseDownInButton)
		addbuttonclicked(event);

	event.stopPropagation();
	event.preventDefault();
}

function findIndexOfChild (child)
{
	var node = child.parentNode;
	
	var i = 0;
	
	for (node = node.firstChild; node != null; node = node.nextSibling, ++i)
	{
		if (node == child)
			return i;
	}
	
	return -1;
}

function removeAllChildren (parent)
{
	while (parent.hasChildNodes())
		parent.removeChild(parent.firstChild);
}

function removeButtonClicked ()
{
	if (selectedRow != null)
	{
		var index = findIndexOfChild (selectedRow);
		var parentNode = selectedRow.parentNode;
		
		symbolData.splice (index, 1);
		allTheData.splice (index, 1);
		parentNode.removeChild(selectedRow);

		selectedRow = null;
		if (allTheData.length > 1)
		{
			if (index >= allTheData.length)
				index = allTheData.length-1;
			
			var nextSelection = parentNode.firstChild;
			for (var i=0; i<index;++i)
				nextSelection = nextSelection.nextSibling;
			
			if (nextSelection != null)
				clickonbackrow (null, nextSelection);
		}
		else
		{
			removeButton.setEnabled(false);
		}
		anythingChangedOnBackside = true;
	}
}

function addsymbol (symbol, name)
{
	var innerList = document.getElementById('inner-list');
	
	var row = document.createElement('div');
	row.setAttribute("class", "row");
	row.setAttribute("onclick", "clickonbackrow(event, this);");
	
	var span = document.createElement('span');
	span.setAttribute ("class", "col1");
	span.innerText = symbol;
	row.appendChild(span);
	
	span = document.createElement('span');
	span.setAttribute ("class", "col2");
	span.innerText = name;
	row.appendChild(span);
	
	innerList.appendChild(row);
	
	selectAllInSymbolTextField();

	return row;
}

function selectAllInSymbolTextField() 
{
	var input = document.getElementById("symbolinput");
	input.blur();
	input.focus();
	input.select();
}

function calculateWidgetHeight() {
	var height = 34 + // table top
	             29 + // table bottom
	             35;  // footer
	
	if (showingCharts)
		height += 80;
		
	var c = allTheData.length - 2;
	
	if (c > 0)
		height += (c * 25); // 25 row height
	
	return height;
}

function onremove ()
{
	// remove all the pref pref keys here
	widget.setPreferenceForKey (null, createKey("symbols"));
	widget.setPreferenceForKey (null, createKey("selection"));
	widget.setPreferenceForKey (null, createKey("chartmode"));
}

function createKey (key)
{
	return widget.identifier + "-" + key
}

function setPreferenceForKey (pref, key)
{
	// set both the per widget pref and the global
	widget.setPreferenceForKey(pref, createKey(key));
	widget.setPreferenceForKey(pref, key);
}

function getPreferenceForKey (key, global)
{
	// first check to see if we have a per instance one
	var pref = widget.preferenceForKey(createKey(key));
	
	if (pref == null && global)
	{
		// now check the global one
		pref = widget.preferenceForKey(key);
	}
	
	return pref;
}

function createAllDataJSString ()
{
	var str = "["
	var c = allTheData.length;
	
	for (var i = 0; i < c; ++i)
	{
		var data = allTheData[i];
		if (i != 0)
			str += ",";
		// make sure the name does not have any single quotes in it
		var name = data.name.replace(/\'/g, "\\'");
		str += "{symbol:'" + data.symbol + "',exchange:'" + data.exchange + "',name:'" + name + "'}";
	}
	
	str += "];";
	
	return str;
}


var resizeAnimation = {startTime:0, duration:250, positionFrom:0, positionTo:0, positionNow:0, timer:null, element:null, onfinished:null};


function limit_3 (a, b, c) {
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease) {
    return from + (to - from) * ease;
}

function animate () {
	var T;
	var ease;
	var time  = (new Date).getTime();
	var yLoc;
	var frame;
		
	T = limit_3(time-resizeAnimation.startTime, 0, resizeAnimation.duration);
	ease = 0.5 - (0.5 * Math.cos(Math.PI * T / resizeAnimation.duration));

	if (T >= resizeAnimation.duration)
	{
		yLoc = resizeAnimation.positionTo;
		clearInterval (resizeAnimation.timer);
		resizeAnimation.timer = null;
		
		if (resizeAnimation.onfinished)
			setTimeout ("resizeAnimation.onfinished();", 0); // call after the last frame is drawn
	}
	else
		yLoc = computeNextFloat(resizeAnimation.positionFrom, resizeAnimation.positionTo, ease);
		
	// convert to a integer, not sure if this is the best way
	resizeAnimation.positionNow = parseInt(yLoc);
	resizeAnimation.element.style.height = resizeAnimation.positionNow + "px";
}

function animFinished()
{
	if (window.widget)
	{
		window.resizeTo (213, calculateWidgetHeight());	
	}
}

var currentChartSelect = null;
var lastChartRequest = null;
function dayclick(event, td, tag)
{
	if (td == currentChartSelect && td != null) return;
	
	if (currentChartSelect != null)
		currentChartSelect.setAttribute ("class", "");
	
	if (currentChartSelect != td)
	{
		td.setAttribute ("class", "selected");
		currentChartSelect = td;
	
		currentChartMode = tag;
		getChartForCurrentMode();
		
		if (window.widget)
			widget.setPreferenceForKey(tag, createKey("chartmode"));
	}
}

function getChartForCurrentMode ()
{
	if (lastChartRequest != null)
		lastChartRequest.abort();
	
	var tag = currentSelection.getAttribute("tag");
	var exchange = currentSelection.getAttribute("stock_exchange");
	lastChartRequest = fetchChartData (generateSymbolStringFromStrings(tag, exchange), currentChartMode, chart_callback);
	enableChart(false);
}

var horizonalLines = new Image();
horizonalLines.src = 'Images/stocks_chartgradient.png';

function addElementToVerticalAxes (vaxes, showPrecision, value)
{
	var div = document.createElement('div');
	if (showPrecision)
		div.innerText = value.toFixed(2).toString();
	else
		div.innerText = Math.round(value);
	vaxes.appendChild (div);
}

function addVerticalAxes (min, max)
{
	var vaxes = document.getElementById('vertical-axes');
	removeAllChildren (vaxes);
		
	var unitdelta = (max - min) * 0.375; // 3/8 for the middle pieces
	var showPrecision = unitdelta <= 1;
	
	// top
	addElementToVerticalAxes (vaxes, showPrecision, max);
	// top middle
	addElementToVerticalAxes (vaxes, showPrecision, max - unitdelta);
	// bottom middle
	addElementToVerticalAxes (vaxes, showPrecision, min + unitdelta);
	//bottom
	addElementToVerticalAxes (vaxes, showPrecision, min);
}

function chart_callback (obj)
{
	if (!obj.error)
	{
		var data = obj.data;
		gChartData = data;
		var c = data.length;

		if (obj.data.length > 0)
		{
		
//
// add the correct vertical lines
//
			var src;
			switch (currentChartMode)
			{
				case "1d": src = 'Images/stocks_7line.png'; break;
				case "1w": src = 'Images/stocks_5line.png'; break;
				case "1m": src = 'Images/stocks_4line.png'; break;
				case "3m": src = 'Images/stocks_3line.png'; break;
				default: src = 'Images/stocks_6line.png'; break;
			}
			document.getElementById('chart-grid').src = src;
			
			enableChart(true);			
			
			
//
// calculate min/max
//

			var min = data[0].close;
			var max = min;
			
			for (var i = 0; i < c; ++i)
			{
				var close = data[i].close;
				if (close > max)
					max = close;
				else if (close < min)
					min = close;
			}
						
			// pad 1/16 of the data
			// turn padding off.
			/*var delta = (max - min)  / 16;
			max = max + delta;
			min = min - delta;
			*/
			var xy = createXYList(min, max, currentChartMode, data);
			gXY = xy;
			c = xy.length;
//
// setup the veritcal axes
//
			addVerticalAxes (min, max);
			setupHorizontalAxes (data	, currentChartMode);	
//
// now clip an draw
//
			
			var context = document.getElementById('chart-canvas').getContext('2d');
						
			context.clearRect(0, 0, kChartCanvasWidth, kChartCanvasHeight);
//
// first clip
//
			
			if (c < 2) // just draw the background
				context.drawImage (horizonalLines, 0, 0);
			else
			{
				context.beginPath();
				context.save();
				context.moveTo(0, kChartCanvasHeight);
				context.lineTo(0, xy[0][1]);
				var val;
				for (var i = 0; i < c; ++i)
				{
					val = xy[i];
					context.lineTo (val[0],val[1]);
				}
				context.lineTo (val[0], kChartCanvasHeight);
				context.clip();
				
				context.drawImage (horizonalLines, 0, 0);
				
				context.restore();

//
// then render line
//
				context.beginPath();
				context.lineWidth = 2.0;
				context.strokeStyle = "rgb(255, 255, 255)";
				
				context.moveTo(0, xy[0][1]);
				for (var i = 0; i < c; ++i)
				{
					val = xy[i];
					context.lineTo (val[0],val[1]);
				}
				context.stroke();
			}
		}
		else
		{
			document.getElementById('retrieving').innerText = getLocalizedString('No chart data');
		}
	}
	else
	{
		enableChart(false);
		document.getElementById('retrieving').innerText = getLocalizedString('Error retrieving chart');
		alert ('fetching chart failed: ' + obj.errorString);
	}
}

// workaround for canvas not rendering after display change.
function enableChart (enable)
{
	var chartgrid = document.getElementById('chart-grid');
	var verticalaxes = document.getElementById('vertical-axes');
	var haxes = document.getElementById('horizontal-axes');
	var retrieving = document.getElementById('retrieving');
	
	if (enable)
	{
		chartgrid.style.display='inline';
		verticalaxes.style.display = 'block';
		haxes.style.display = 'block';
		
		retrieving.style.display='none';
	}
	else
	{
		retrieving.innerText = getLocalizedString('Retrieving Data…');
		retrieving.style.display='block';

		chartgrid.style.display='none';
		verticalaxes.style.display = 'none';
		haxes.style.display = 'none';
		
		// ick ick ick, clear the canvas
		document.getElementById('chart-canvas').getContext('2d').clearRect (0, 0, kChartCanvasWidth, kChartCanvasHeight);
	}
}

function setupChartController (mode)
{
	var array = ['1d', '1w', '1m', '3m', '6m', '1y', '2y'];
	var c = array.length;
	
	for (var i = 0; i < c; ++i)
	{
		var m = array[i];
		var div = document.getElementById(m+'text');
		div.innerHTML = getLocalizedString(m);
		
		if (m == mode)
		{
			if (currentChartSelect != null)
				currentChartSelect.setAttribute ("class", "");
			
			currentChartSelect = div.parentNode;
			currentChartSelect.setAttribute ("class", "selected");
		}
	}
}


function getReverseMonthArray(startingAt)
{
	var array = getLocalizedString("Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec").split(" ");
	
	if (startingAt < 11)
	{
		var a2 = array.splice (0, startingAt + 1);
		
		array = array.concat(a2);
	}
	
	array.reverse();
		
	return array;
}

function setupHorizontalAxes (chartData, mode)
{
	var xaxes = document.getElementById('horizontal-axes');
	
	removeAllChildren (xaxes);
	
	switch (mode)
	{
		case "1d":
			var items = [10, 11, 12, 1, 2, 3, 4];
						
			for (var i = 0; i < 7; ++i)
			{
				var div = document.createElement('div');
				div.innerText = items[i];
				div.setAttribute ("class", "xaxes7 xaxes7-"+i);
				xaxes.appendChild(div);
			}
			break;
			
		case "1w":
            var items = new Array();

            var i = 0;
            for (var index = chartData.length - 1; index >= 0; index--)
            {                
                var date = chartData[index].date;
                var day = date.getDate();
                if (i == 0 || items[i-1] != day)
                {
                    items = items.concat(day);
                    i++;
                }
            }
						
			for (var i = 4, j = 0; i >= 0; --i, ++j)
			{
				var div = document.createElement('div');
				div.innerText = (undefined == items[i] ? "" : items[i]);
				div.setAttribute ("class", "xaxes5 xaxes5-"+j);
				xaxes.appendChild(div);
			}
			break;

		case "1m":
            var numLabels = 4;

            var items = new Array();

            var index = chartData.length-1;
			for (var i = 0; i < numLabels; i++)
            {
                var date = chartData[index].date;
                var day = date.getDate();
                items = items.concat(day);
            
                index -= 5; // go back a week
            }
						
			for (var i = numLabels-1, j = 0; i >= 0; --i, ++j)
			{
				var div = document.createElement('div');
				div.innerText = items[i];
				div.setAttribute ("class", "xaxes4 xaxes4-"+j);
				xaxes.appendChild(div);
			}
			break;

		case "3m":
			var labels = getReverseMonthArray (chartData[chartData.length-1].date.getMonth());

			for (var i = 2, j = 0; i >= 0; --i, ++j)
			{
				var div = document.createElement('div');
				div.innerText = labels[i];
				div.setAttribute ("class", "xaxes3 xaxes3-" + j);
				xaxes.appendChild (div);
			}
			break;

		case "6m":
			var labels = getReverseMonthArray (chartData[chartData.length-1].date.getMonth());
			
			var j=0
			for (var i = 5, j = 0; i >= 0; --i, j++)
			{
				var div = document.createElement('div');
				div.innerText = labels[i];
				div.setAttribute ("class", "xaxes6 xaxes6-"+j);
				xaxes.appendChild (div);
			}
			break;
	
		case "1y":
			var labels = getReverseMonthArray (chartData[chartData.length-1].date.getMonth());

			for (var i = 10, j = 0; i >= 0; i-=2, j++)
			{
				var div = document.createElement('div');
				div.innerText = labels[i];
				div.setAttribute ("class", "xaxes6 xaxes6-"+j);
				xaxes.appendChild (div);
			}
			break;

		case "2y":
			var lastDate = chartData[chartData.length-1].date;
			var labels = getReverseMonthArray (lastDate.getMonth());
			var indexes = [8, 4, 0, 8, 4, 0];
			
			var year = lastDate.getFullYear() -1;
			for (var i = 0; i < 6; ++i)
			{
				var div = document.createElement('div');
				var index = indexes[i];
				
				if (index == 0)
				{
                    // XXX: Localizability
					div.innerText = labels[index].substr(0, 1) + "'" + year.toString().substr(2,2);
					year++;
				}
				else
					div.innerText = labels[index];
				div.setAttribute ("class", "xaxes6 xaxes6-"+i);
				xaxes.appendChild (div);
			}
			break;
	}
}

function createXYList(min, max, mode, data) {
	var pixelsPerY = 46 / (max - min); // 48 = height of graph, 66 inset by 1px

	switch (mode) {
		case "1d": return createXYList1d (min, max, pixelsPerY, data); break;
		case "1w": return createXYListAverage (min, max, pixelsPerY, data); break;
		case "1m": return createXYListAverage (min, max, pixelsPerY, data); break;
		case "3m": return createXYListAverage (min, max, pixelsPerY, data); break;
		case "6m": return createXYListAverage (min, max, pixelsPerY, data); break;
		case "1y": return createXYListAverage (min, max, pixelsPerY, data); break;
		case "2y": return createXYListAverage (min, max, pixelsPerY, data); break;
	}
	
}

function createXYList1d(min, max, pixelsPerY, data) {
	var array = new Array();
	var c = data.length;
	var numOpenHours = 6.5;
	var utcOffset = 13;
	var currx = 0;
	for (var i = 0; i < c; ++i) {
		var close = data[i].close - min; // normalize	
		var date = data[i].date;
		var hour = date.getUTCHours();
		var minutes = date.getUTCMinutes();
		hour -= utcOffset;
		minutes = (minutes / 60) - .5;
		currx = kChartCanvasWidth * ((hour+minutes) / numOpenHours);
		array[i] = [currx, (46 - (pixelsPerY * close)) + 1];
	}
	return array;
}

function createXYListAverage(min, max, pixelsPerY, data) {
	var array = new Array();
	var c = data.length;
	
	var pixelsPerX = kChartCanvasWidth / c;
	var currx = 0;
	
	for (var i = 0; i < c; ++i) {
		
		var close = data[i].close - min; // normalize	        

		array[i] = [currx, (46 - (pixelsPerY * close)) + 1];
		
		currx += pixelsPerX;
	}
	
	array[i-1][0] = kChartCanvasWidth; // make sure it ends at the right
	
	return array;
}

function tosClick(evt) {
	if (window.widget) {
		widget.openURL("http://info.yahoo.com/legal/us/yahoo/utos/utos-173.html");
	}
}

function financeLogoClick(evt) {
	if (window.widget) {
		widget.openURL("http://api.apple.go.yahoo.com/appledwf/");
	}
}

function yLogoClick(evt) {
	if (window.widget) {
		var url = "http://api.apple.go.yahoo.com/appledwf/q/cq?d=v1";
		for (var i = 0; i < allTheData.length; i++) {
			var symbol = allTheData[i].symbol;
			symbol = ("COMPX" == symbol) ? "^IXIC" : symbol;
			url += "&s=" + symbol;
		}
		widget.openURL(url);
	}
}
